/** @file fs.c
*
* @author Sylvain Nahas
*
* This module provides FS related function implementation.
*/

/** @par LICENCE
* @verbatim
 Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code and the documentation.
        
        ADDI-DATA GmbH
        Airpark Business Center
        Airport Boulevard B210
        77836 Rheinmünster
        Germany
        Tel: +49(0)7229/1847-0

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. 
        
    You shoud find the complete GPL in the COPYING file accompanying 
    this source code. 
* @endverbatim
*/

#include "apci3200-private.h"


/**@def EXPORT_NO_SYMBOLS
 * Function in this file are not exported.
 */
EXPORT_NO_SYMBOLS;
//------------------------------------------------------------------------------
/** open() function of the module for the APCI-3200, with lookup though global OS PCI list 
* this is for files unmanaged by the driver itself and use the minor device number to identify the board.
*
* When opening, the pci_dev associated to the minor number is looked up
* and associated with the file structure. It avoid further lookup when calling ioctl()
* 
*
*/
int apci3200_open_lookup (struct inode *inode, struct file *filp)
{
   if ( apci3200_INDEX_NOT_VALID(&apci3200_count, MINOR(inode->i_rdev) ) )
   {
   	return -ENODEV;
   }
      
   filp->private_data = apci3200_lookup_board_by_index(MINOR(inode->i_rdev) );  
   
   MOD_INC_USE_COUNT;
   return 0;   
}
//------------------------------------------------------------------------------
/** release() function of the module for the APCI-3200, with lookup though global OS PCI list 
* this is for files unmanaged by the driver itself and use the minor device number to identify the board
*/
int apci3200_release_lookup (struct inode *inode,struct file *filp)
{

   if ( apci3200_INDEX_NOT_VALID(&apci3200_count, MINOR(inode->i_rdev) ) )
   {
   	return -ENODEV;
   }

   MOD_DEC_USE_COUNT;
   return 0;
}
//------------------------------------------------------------------------------
/** ioctl() function of the module for the APCI-3200, with lookup though global OS PCI list 
* this is for files unmanaged by the driver itself and use the minor device number to identify the board
*/
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,36)
        int apci3200_ioctl_lookup (struct inode *inode,
                                                           struct file *filp,
                                                           unsigned int cmd,
                                                           unsigned long arg)
#else
        long apci3200_ioctl_lookup (struct file *filp,
                                                                unsigned int cmd,
                                                                unsigned long arg)
#endif
        {
                int ret = 0;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36)
                struct inode *inode = filp->f_dentry->d_inode;
#endif

   if ( apci3200_INDEX_NOT_VALID(&apci3200_count, MINOR(inode->i_rdev) ) )
   {
   	return -ENODEV;
   }  
     
   if (!filp->private_data) // private data is initialised to NULL
   {
   	printk(KERN_CRIT "%s: %s: board data should be affected but is not (did you call open() before ioctl() ?) \n",__DRIVER_NAME, __FUNCTION__);
	return -EBADFD;
   }

   return apci3200_do_ioctl( (struct pci_dev*) filp->private_data, cmd,arg);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36)
   lock_kernel();
#endif
           ret = apci3200_do_ioctl( (struct pci_dev*) filp->private_data, cmd, arg);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36)
   unlock_kernel();
#endif
   return ret;

}

